import java.net.*;
import java.io.*;

// http://pirate.shu.edu/~wachsmut/Teaching/CSAS2214/Virtual/Lectures/chat-client-server.html

public class ChatThread extends Thread
{  
	DataInputStream streamIn;
	DataOutputStream streamOut;
	DataInputStream console;
	int peer_pub_key, peer_c, priv_key, c, debug;
	
	public void run() {
		// if in is null, this is a sender thread, else it's a receiver thread
		if(streamIn == null) {
			send();
		} else {
			receive();
		}
	}
	
	public void send() {
		  String line = "";
		  while (!line.equals(".bye"))
		  {  try
			 {  System.out.println("Type, enter .bye to quit");
				line = console.readLine();
				String encline = encryptstring(line);
				streamOut.writeUTF(encline);
				streamOut.flush();
			 }
			 catch(IOException ioe)
			 {  System.out.println("Sending error: " + ioe.getMessage());
			 }
		  }	
	}
	
	public void receive() {
		 boolean done = false;
         while (!done)
         {  try
            {  String line = streamIn.readUTF();
               if(debug == 1) {
                   System.out.println("RAW RECEIVED: " + line);
               }
               String decline = decryptstring(line);
               System.out.println(decline);
               done = decline.equals(".bye");
            }
            catch(IOException ioe)
            {  done = true;
            }
         }	
	}
	
	// given the keys, encrypt line using the peer keys and generate a space-separated string of encrypted numbers, each number corresponding to a character.  If debug, print the resulting encrypted string
	public String encryptstring(String line) {
		String result = "";
		
		for(int i = 0; i < line.length(); i++) {
			char ch = line.charAt(i);
			int asciic = (int)ch;
			
			Integer encryptedc = new Integer(Util.endecrypt(asciic, peer_pub_key, peer_c));
			
			if(i > 0) {
				result = result + " ";
			}
			
			result = result + encryptedc.toString();
		}
		
		if(debug == 1) {
			System.out.println("ENCRYPTED " + line + " TO " + result);
		}
		
		return result;
	}
	
	// given a string of space-delimited encrypted numbers, decrypt each number into ASCII, convert to a char, and append to the resulting string.  If debug, print the received encrypted string
	public String decryptstring(String line) {
		String result = "";
		
		String[] encchars = line.split(" ");
		
		for(int i = 0; i < encchars.length; i++) {
			Integer encryptedc = Integer.parseInt(encchars[i]);
			
			int asciic = Util.endecrypt(encryptedc.intValue(), priv_key, c);
			
			char ch = (char) asciic;
			
			result = result + ch;
		}
		
		if(debug == 1) {
			System.out.println("DECRYPTED " + line + " TO " + result);
		}
		
		return result;
	}
	
	public ChatThread(DataInputStream _in, DataOutputStream _out, DataInputStream _console, int _peer_pub_key, int _peer_c, int _priv_key, int _c, int _debug) {
		streamIn = _in;
		streamOut = _out;
		console = _console;
		
	    peer_pub_key = _peer_pub_key;
        peer_c = _peer_c;
        priv_key = _priv_key;
        c = _c;
        debug = _debug;		
	}
}